# mp3player.py
# workes with RC522 @ 13,56MHz
from dfplayer import DFPlayer
import mfrc522
# from os import uname
from machine import Pin, SoftI2C, SPI, ADC, reset
# from oled import OLED # Vorsicht: GND Vcc SCK SDA
from oled_SH1106 import OLED # Vorsicht: Vdd GND SCK SDA
from time import sleep
from sys import exit
from timeout import *

# Initialisieren der Schnittstellen **********************
vol=ADC(Pin(36))
vol.atten(vol.ATTN_11DB)
vol.width(vol.WIDTH_10BIT)

RX_Pin = 16
TX_Pin = 17
busyPin = 27
df=DFPlayer(busyPinNbr=busyPin,
            txd=TX_Pin,
            rxd=RX_Pin,
            volume=15)
sleep(2) # wait for player initializing
numOfTitles=df.getNumberOfFiles()
print("Titel gesamt:",numOfTitles)

spi=SPI(2,baudrate=3200000)
#                         cs=sda
rdr = mfrc522.MFRC522(spi, cs=5, baudrate=3200000)

i2c=SoftI2C(scl=Pin(22),sda=Pin(21))
d=OLED(i2c)

MasterID=4217116188  #  0XFB5C161C

listen=[
        "COUNTRY",
        "BLACKMOORSNIGHT",
        "YOUTUBE",
        "SCHLAGER",
        "HITS",
        "VON CD"
        "OLDIES"
       ]

# Funktionen defnieren ***********************************
def readUID(display,kartentyp,timeout): 
    global song
    display.clearFT(0,1,15,2,False)
    display.writeAt("PUT ON",0,1,False)
    display.writeAt(kartentyp,0,2)
    readTimeOut=TimeOutMs(timeout)
    while not readTimeOut():
        (stat, tag_type) = rdr.request(rdr.REQIDL)
        if stat == rdr.OK:
            (stat, raw_uid) = rdr.anticoll()
            if stat == rdr.OK:
                display.clearFT(0,3,15,3)
                display.writeAt("Card OK",0,3)
                userID=0
                for i in range(4):
                    userID=(userID<<8) | raw_uid[i]
                userIDS="{:#X}".format(userID)
                print(userIDS)
                display.writeAt(userIDS+"     ",0,4)
                sleep(2)
                df.reset()
                song=0
                return userID,userIDS
    return None

def addUID(display):
    display.clearAll()
    m=readUID(display,"Master",3000)
    print("Master",m)
    if m is not None:
        mid,mids= m
        if mid==MasterID:
            sleep(5)
            print("Master OK")
            u=readUID(display,"Slavecard",3000)
            if u is not None:
                uid,uids=u
                if uid is not None and uid != MasterID:
                    with open("slavecards.txt","a") as f:
                        f.write("{}\n".format(uids))
                    display.writeAt("New slave written",0,4)
                    display.writeAt("{}".format(uids),0,5)
                    sleep(5)
                    return True
            else:
                display.writeAt("ERROR!!!",0,3)
                display.writeAt("Card not added!",0,4)
                return False
        else:
            display.writeAt("ERROR!!!",0,3)
            display.writeAt("Not mastercard",0,4)
            sleep(3)
    return False

def stopDisplay():
    d.clearAll()
    d.writeAt("GOOD BYE",4,2)
    exit()

def readTags():
    tags=[]
    with open("slavecards.txt","r") as f:
        for line in f:
            tags.append(line.strip("\n"))
    if tags:
        return tags
    else:
        return None

def getNumberOfTitels():
    fileCount=[]
    nof=df.getNumberOfFolders()
    df.volume(0)
    sleep(0.2)
    for t in range(nof):
        df.play(t,0)
        sleep(0.2)
        df.stop()
        sleep(0.2)
        n=df.getNumberOfFilesInFolder()
        print(t,n)
        d.writeAt("Track:{}".format(t),0,4,False)
        d.writeAt("Songs:{}".format(n),0,5)
        fileCount.append(n)
    df.stop()
    volume=int(vol.read()*100/1024)
    df.volume(volume)
    sleep(0.2)
    return fileCount

def player(d,t,nof):
    global acard
    for s in range(nof):
        print (t, s, df.isPlaying(),nof)
        if not df.isPlaying():
            df.play(t,s)
            d.clearFT(0,5,15,5,False)
            d.writeAt("Titel: {}".format(s),0,5)
            while df.isPlaying():
                volume=int(vol.read()*100/1024)
                df.volume(volume)
                u=readUID(d,"TAG TO STOP",100)
                if u is not None:
                    d.clearFT(0,3,15)
                    print("Stoped with",u[1])
                    df.stop()
                    acard=""
                    ncard=u[1]
                    return
    d.clearFT(0,3,15)
    df.stop()
    

# ******************* Hauptprogramm *********************
d.clearAll()

cards=[]
acard=""
ncard=""
song=0
n=0
try:
    cards=readTags()
except OSError as e:
    print("keine Datei, keine Daten!")
    allRead=TimeOutMs(10000)
    while not allRead():
        if addUID(d):
            allRead=TimeOutMs(10000)
    print("Alle Karten eingelesen und gespeichert")
    d.clearFT(0,3,15,4,False)
    d.writeAt(" ALL CARDS READ",0,3)
    d.writeAt("**R E B O O T**",0,4)
#     reset()
    cards=readTags()
    
if cards:
    d.writeAt("***MP3-PLAYER***",0,0,False)
    d.writeAt("PUT ON MP3-TAG",0,1,False)
    d.writeAt("Hole die Anzahl",0,2,False)
    d.writeAt("Songs/Folder",0,3)
    filesInFolder=getNumberOfTitels()
    d.clearFT(0,2,15)
    df.stop()
    while 1:
        uid=readUID(d,"PLAYLIST-TAG",1000)
        if uid is not None:
            ncard=uid[1]
        volume=int(vol.read()*100/1024)
        df.volume(volume)
        if acard != ncard:
            print("in",acard,ncard)
            acard=ncard
            if acard in cards:
                d.clearFT(0,2,15,5, False)
                t=cards.index(acard) # Ordnernummer
                song=0
                print("PLAYLIST {}".format(t))
                print(listen[t])
                d.writeAt("PLAYLIST {}".format(t),0,3,False)
                d.writeAt(listen[t],0,4)
                print("playing",df.isPlaying())
                print("Start",t,song,filesInFolder[t],
                          df.isPlaying())
                player(d,t,filesInFolder[t])
                print("back in main")
            else:
                d.clearFT(0,3,15,4, False)
                d.writeAt("PLAYLIST TAG",1,3,False)
                d.writeAt(" NOT FOUND",1,4)
                t=-1
                sleep(3)
